@@ -205,7 +205,7 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      g_cclosure_marshal_generic,
 			      G_TYPE_NONE,
 			      0);
 
@@ -428,6 +428,7 @@ empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
 
 	data.chooser = chooser;
 	data.account = account;
+	data.set = FALSE;
 
 	gtk_tree_model_foreach (model,
 				(GtkTreeModelForeachFunc) account_chooser_set_account_foreach,
@@ -569,7 +570,7 @@ account_manager_prepared_cb (GObject *source_object,
 	EmpathyAccountChooserPriv *priv = GET_PRIV (chooser);
 	GError *error = NULL;
 
-	if (!tp_account_manager_prepare_finish (manager, result, &error)) {
+	if (!tp_proxy_prepare_finish (manager, result, &error)) {
 		DEBUG ("Failed to prepare account manager: %s", error->message);
 		g_error_free (error);
 		return;
@@ -685,7 +686,7 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 					NULL);
 
 	/* Populate accounts */
-	tp_account_manager_prepare_async (priv->manager, NULL,
+	tp_proxy_prepare_async (priv->manager, NULL,
 					  account_manager_prepared_cb, chooser);
 
 	g_object_unref (store);
