@@ -664,7 +664,7 @@ empathy_individual_menu_class_init (EmpathyIndividualMenuClass *klass)
   signals[SIGNAL_LINK_CONTACTS_ACTIVATED] =
       g_signal_new ("link-contacts-activated", G_OBJECT_CLASS_TYPE (klass),
           G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-          g_cclosure_marshal_VOID__OBJECT,
+          g_cclosure_marshal_generic,
           G_TYPE_NONE, 1, EMPATHY_TYPE_LINKING_DIALOG);
 
   g_type_class_add_private (object_class, sizeof (EmpathyIndividualMenuPriv));
@@ -786,7 +786,8 @@ empathy_individual_sms_menu_item_activated (GtkMenuItem *item,
   empathy_sms_contact_id (
       empathy_contact_get_account (contact),
       empathy_contact_get_id (contact),
-      empathy_get_current_action_time ());
+      empathy_get_current_action_time (),
+      NULL, NULL);
 }
 
 GtkWidget *
@@ -1131,8 +1132,9 @@ empathy_individual_edit_menu_item_new (FolksIndividual *individual)
       manager = empathy_individual_manager_dup_singleton ();
       connection = empathy_contact_get_connection (contact);
 
-      enable = (empathy_connection_can_alias_personas (connection) &&
-                empathy_connection_can_group_personas (connection));
+      enable = (empathy_connection_can_alias_personas (connection,
+						       individual) &&
+                empathy_connection_can_group_personas (connection, individual));
 
       g_object_unref (manager);
     }
