@@ -73,22 +73,12 @@ typedef struct
   GHashTable *status_icons;
   /* List of owned GCancellables for each pending avatar load operation */
   GList *avatar_cancellables;
+  /* Hash: FolksIndividual* -> GQueue (GtkTreeIter *) */
+  GHashTable                  *folks_individual_cache;
+  /* Hash: char *groupname -> GtkTreeIter * */
+  GHashTable                  *empathy_group_cache;
 } EmpathyIndividualStorePriv;
 
-typedef struct
-{
-  GtkTreeIter iter;
-  const gchar *name;
-  gboolean found;
-} FindGroup;
-
-typedef struct
-{
-  FolksIndividual *individual;
-  gboolean found;
-  GList *iters;
-} FindContact;
-
 typedef struct
 {
   EmpathyIndividualStore *self;
@@ -115,55 +105,6 @@ static void individual_store_contact_update (EmpathyIndividualStore *self,
 G_DEFINE_TYPE (EmpathyIndividualStore, empathy_individual_store,
     GTK_TYPE_TREE_STORE);
 
-/* Calculate whether the Individual can do audio or video calls.
- * FIXME: We can remove this once libfolks has grown capabilities support
- * again: bgo#626179. */
-static void
-individual_can_audio_video_call (FolksIndividual *individual,
-    gboolean *can_audio_call,
-    gboolean *can_video_call)
-{
-  GeeSet *personas;
-  GeeIterator *iter;
-  gboolean can_audio = FALSE, can_video = FALSE;
-
-  personas = folks_individual_get_personas (individual);
-  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
-  while (gee_iterator_next (iter))
-    {
-      FolksPersona *persona = gee_iterator_get (iter);
-      TpContact *tp_contact;
-
-      if (!empathy_folks_persona_is_interesting (persona))
-        goto while_finish;
-
-      tp_contact = tpf_persona_get_contact (TPF_PERSONA (persona));
-      if (tp_contact != NULL)
-        {
-          EmpathyContact *contact;
-
-          contact = empathy_contact_dup_from_tp_contact (tp_contact);
-          empathy_contact_set_persona (contact, persona);
-
-          can_audio = can_audio || empathy_contact_get_capabilities (contact) &
-              EMPATHY_CAPABILITIES_AUDIO;
-          can_video = can_video || empathy_contact_get_capabilities (contact) &
-              EMPATHY_CAPABILITIES_VIDEO;
-
-          g_object_unref (contact);
-        }
-while_finish:
-      g_clear_object (&persona);
-
-      if (can_audio && can_video)
-        break;
-    }
-  g_clear_object (&iter);
-
-  *can_audio_call = can_audio;
-  *can_video_call = can_video;
-}
-
 static const gchar * const *
 individual_get_client_types (FolksIndividual *individual)
 {
@@ -212,11 +153,13 @@ add_individual_to_store (GtkTreeStore *self,
     GtkTreeIter *parent,
     FolksIndividual *individual)
 {
+  EmpathyIndividualStorePriv *priv = GET_PRIV (self);
   gboolean can_audio_call, can_video_call;
   const gchar * const *types;
+  GQueue *queue;
 
-  individual_can_audio_video_call (individual, &can_audio_call,
-      &can_video_call);
+  empathy_individual_can_audio_video_call (individual, &can_audio_call,
+      &can_video_call, NULL);
 
   types = individual_get_client_types (individual);
 
@@ -230,34 +173,19 @@ add_individual_to_store (GtkTreeStore *self,
       EMPATHY_INDIVIDUAL_STORE_COL_CAN_VIDEO_CALL, can_video_call,
       EMPATHY_INDIVIDUAL_STORE_COL_CLIENT_TYPES, types,
       -1);
-}
 
-static gboolean
-individual_store_get_group_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    FindGroup *fg)
-{
-  gchar *str;
-  gboolean is_group;
-
-  /* Groups are only at the top level. */
-  if (gtk_tree_path_get_depth (path) != 1)
-    return FALSE;
-
-  gtk_tree_model_get (model, iter,
-      EMPATHY_INDIVIDUAL_STORE_COL_NAME, &str,
-      EMPATHY_INDIVIDUAL_STORE_COL_IS_GROUP, &is_group, -1);
-
-  if (is_group && !tp_strdiff (str, fg->name))
+  queue = g_hash_table_lookup (priv->folks_individual_cache, individual);
+  if (queue)
     {
-      fg->found = TRUE;
-      fg->iter = *iter;
+      g_queue_push_tail (queue, gtk_tree_iter_copy (iter));
+    }
+  else
+    {
+      queue = g_queue_new ();
+      g_queue_push_tail (queue, gtk_tree_iter_copy (iter));
+      g_hash_table_insert (priv->folks_individual_cache, individual,
+          queue);
     }
-
-  g_free (str);
-
-  return fg->found;
 }
 
 static void
@@ -268,20 +196,16 @@ individual_store_get_group (EmpathyIndividualStore *self,
     gboolean *created,
     gboolean is_fake_group)
 {
+  EmpathyIndividualStorePriv *priv = GET_PRIV (self);
   GtkTreeModel *model;
   GtkTreeIter iter_group;
   GtkTreeIter iter_separator;
-  FindGroup fg;
-
-  memset (&fg, 0, sizeof (fg));
-
-  fg.name = name;
+  GtkTreeIter *iter;
 
   model = GTK_TREE_MODEL (self);
-  gtk_tree_model_foreach (model,
-      (GtkTreeModelForeachFunc) individual_store_get_group_foreach, &fg);
+  iter = g_hash_table_lookup (priv->empathy_group_cache, name);
 
-  if (!fg.found)
+  if (iter == NULL)
     {
       if (created)
         *created = TRUE;
@@ -296,6 +220,9 @@ individual_store_get_group (EmpathyIndividualStore *self,
           EMPATHY_INDIVIDUAL_STORE_COL_IS_FAKE_GROUP, is_fake_group,
           -1);
 
+      g_hash_table_insert (priv->empathy_group_cache, g_strdup (name),
+          gtk_tree_iter_copy (&iter_group));
+
       if (iter_group_to_set)
         *iter_group_to_set = iter_group;
 
@@ -313,9 +240,9 @@ individual_store_get_group (EmpathyIndividualStore *self,
         *created = FALSE;
 
       if (iter_group_to_set)
-        *iter_group_to_set = fg.iter;
+        *iter_group_to_set = *iter;
 
-      iter_separator = fg.iter;
+      iter_separator = *iter;
 
       if (gtk_tree_model_iter_next (model, &iter_separator))
         {
@@ -330,48 +257,28 @@ individual_store_get_group (EmpathyIndividualStore *self,
     }
 }
 
-static gboolean
-individual_store_find_contact_foreach (GtkTreeModel *model,
-    GtkTreePath *path,
-    GtkTreeIter *iter,
-    FindContact *fc)
-{
-  FolksIndividual *individual;
-
-  gtk_tree_model_get (model, iter,
-      EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL, &individual, -1);
-
-  if (individual == fc->individual)
-    {
-      fc->found = TRUE;
-      fc->iters = g_list_append (fc->iters, gtk_tree_iter_copy (iter));
-    }
-
-  tp_clear_object (&individual);
-
-  return FALSE;
-}
-
 static GList *
 individual_store_find_contact (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
-  GtkTreeModel *model;
-  GList *l = NULL;
-  FindContact fc;
+  EmpathyIndividualStorePriv *priv = GET_PRIV (self);
+  GQueue *row_refs_queue;
+  GList *i;
+  GList *iters_list = NULL;
 
-  memset (&fc, 0, sizeof (fc));
+  row_refs_queue = g_hash_table_lookup (priv->folks_individual_cache,
+      individual);
+  if (!row_refs_queue)
+    return NULL;
 
-  fc.individual = individual;
+  for (i = g_queue_peek_head_link (row_refs_queue) ; i != NULL ; i = i->next)
+    {
+      GtkTreeIter *iter = i->data;
 
-  model = GTK_TREE_MODEL (self);
-  gtk_tree_model_foreach (model,
-      (GtkTreeModelForeachFunc) individual_store_find_contact_foreach, &fc);
+      iters_list = g_list_prepend (iters_list, gtk_tree_iter_copy (iter));
+    }
 
-  if (fc.found)
-    l = fc.iters;
-
-  return l;
+  return iters_list;
 }
 
 static void
@@ -385,36 +292,45 @@ static void
 individual_store_remove_individual (EmpathyIndividualStore *self,
     FolksIndividual *individual)
 {
+  EmpathyIndividualStorePriv *priv = GET_PRIV (self);
   GtkTreeModel *model;
-  GList *iters, *l;
+  GQueue *row_refs;
+  GList *l;
 
-  iters = individual_store_find_contact (self, individual);
-  if (iters == NULL)
+  row_refs = g_hash_table_lookup (priv->folks_individual_cache, individual);
+  if (!row_refs)
     return;
 
   /* Clean up model */
   model = GTK_TREE_MODEL (self);
 
-  for (l = iters; l; l = l->next)
+  for (l = g_queue_peek_head_link (row_refs); l; l = l->next)
     {
+      GtkTreeIter *iter = l->data;
       GtkTreeIter parent;
 
       /* NOTE: it is only <= 2 here because we have
        * separators after the group name, otherwise it
        * should be 1.
        */
-      if (gtk_tree_model_iter_parent (model, &parent, l->data) &&
+      if (gtk_tree_model_iter_parent (model, &parent, iter) &&
           gtk_tree_model_iter_n_children (model, &parent) <= 2)
         {
+          gchar *group_name;
+          gtk_tree_model_get (model, &parent,
+              EMPATHY_CONTACT_LIST_STORE_COL_NAME, &group_name,
+              -1);
+          g_hash_table_remove (priv->empathy_group_cache,
+              group_name);
           gtk_tree_store_remove (GTK_TREE_STORE (self), &parent);
         }
       else
         {
-          gtk_tree_store_remove (GTK_TREE_STORE (self), l->data);
+          gtk_tree_store_remove (GTK_TREE_STORE (self), iter);
         }
     }
 
-  free_iters (iters);
+  g_hash_table_remove (priv->folks_individual_cache, individual);
 }
 
 static void
@@ -805,8 +721,8 @@ individual_store_contact_update (EmpathyIndividualStore *self,
       gboolean can_audio_call, can_video_call;
       const gchar * const *types;
 
-      individual_can_audio_video_call (individual, &can_audio_call,
-          &can_video_call);
+      empathy_individual_can_audio_video_call (individual, &can_audio_call,
+          &can_video_call, NULL);
 
       types = individual_get_client_types (individual);
 
@@ -952,6 +868,20 @@ individual_personas_changed_cb (FolksIndividual *individual,
   g_clear_object (&iter);
 }
 
+static void
+individual_store_favourites_changed_cb (FolksIndividual *individual,
+    GParamSpec *param,
+    EmpathyIndividualStore *self)
+{
+  DEBUG ("Individual %s is %s a favourite",
+      folks_individual_get_id (individual),
+      folks_favourite_details_get_is_favourite (
+        FOLKS_FAVOURITE_DETAILS (individual)) ? "now" : "no longer");
+
+  individual_store_remove_individual (self, individual);
+  individual_store_add_individual (self, individual);
+}
+
 void
 individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
     FolksIndividual *individual)
@@ -970,6 +900,8 @@ individual_store_add_individual_and_connect (EmpathyIndividualStore *self,
       (GCallback) individual_store_individual_updated_cb, self);
   g_signal_connect (individual, "personas-changed",
       (GCallback) individual_personas_changed_cb, self);
+  g_signal_connect (individual, "notify::is-favourite",
+      (GCallback) individual_store_favourites_changed_cb, self);
 
   /* provide an empty set so the callback can assume non-NULL sets */
   individual_personas_changed_cb (individual,
@@ -992,6 +924,8 @@ individual_store_disconnect_individual (EmpathyIndividualStore *self,
       (GCallback) individual_store_individual_updated_cb, self);
   g_signal_handlers_disconnect_by_func (individual,
       (GCallback) individual_personas_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (individual,
+      (GCallback) individual_store_favourites_changed_cb, self);
 }
 
 void
@@ -1028,20 +962,6 @@ individual_store_members_changed_cb (EmpathyIndividualManager *manager,
     }
 }
 
-static void
-individual_store_favourites_changed_cb (EmpathyIndividualManager *manager,
-    FolksIndividual *individual,
-    gboolean is_favourite,
-    EmpathyIndividualStore *self)
-{
-  DEBUG ("Individual %s is %s a favourite",
-      folks_individual_get_id (individual),
-      is_favourite ? "now" : "no longer");
-
-  individual_store_remove_individual (self, individual);
-  individual_store_add_individual (self, individual);
-}
-
 static void
 individual_store_groups_changed_cb (EmpathyIndividualManager *manager,
     FolksIndividual *individual,
@@ -1084,10 +1004,6 @@ individual_store_manager_setup (gpointer user_data)
       "members-changed",
       G_CALLBACK (individual_store_members_changed_cb), self);
 
-  g_signal_connect (priv->manager,
-      "favourites-changed",
-      G_CALLBACK (individual_store_favourites_changed_cb), self);
-
   g_signal_connect (priv->manager,
       "groups-changed",
       G_CALLBACK (individual_store_groups_changed_cb), self);
@@ -1166,8 +1082,6 @@ individual_store_dispose (GObject *object)
       G_CALLBACK (individual_store_member_renamed_cb), object);
   g_signal_handlers_disconnect_by_func (priv->manager,
       G_CALLBACK (individual_store_members_changed_cb), object);
-  g_signal_handlers_disconnect_by_func (priv->manager,
-      G_CALLBACK (individual_store_favourites_changed_cb), object);
   g_signal_handlers_disconnect_by_func (priv->manager,
       G_CALLBACK (individual_store_groups_changed_cb), object);
   g_object_unref (priv->manager);
@@ -1183,6 +1097,8 @@ individual_store_dispose (GObject *object)
     }
 
   g_hash_table_destroy (priv->status_icons);
+  g_hash_table_destroy (priv->folks_individual_cache);
+  g_hash_table_destroy (priv->empathy_group_cache);
   G_OBJECT_CLASS (empathy_individual_store_parent_class)->dispose (object);
 }
 
@@ -1619,6 +1535,14 @@ individual_store_inhibit_active_cb (EmpathyIndividualStore *self)
   return FALSE;
 }
 
+static void
+g_queue_free_full_iter (gpointer data)
+{
+  GQueue *queue = (GQueue *) data;
+  g_queue_foreach (queue, (GFunc) gtk_tree_iter_free, NULL);
+  g_queue_free (queue);
+}
+
 static void
 empathy_individual_store_init (EmpathyIndividualStore *self)
 {
@@ -1634,6 +1558,10 @@ empathy_individual_store_init (EmpathyIndividualStore *self)
       (GSourceFunc) individual_store_inhibit_active_cb, self);
   priv->status_icons =
       g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+  priv->folks_individual_cache = g_hash_table_new_full (NULL, NULL, NULL,
+      g_queue_free_full_iter);
+  priv->empathy_group_cache = g_hash_table_new_full (g_str_hash,
+      g_str_equal, g_free, (GDestroyNotify) gtk_tree_iter_free);
   individual_store_setup (self);
 }
 
@@ -1805,6 +1733,10 @@ empathy_individual_store_set_show_groups (EmpathyIndividualStore *self,
       GList *contacts;
 
       gtk_tree_store_clear (GTK_TREE_STORE (self));
+      /* Also clear the cache */
+      g_hash_table_remove_all (priv->folks_individual_cache);
+      g_hash_table_remove_all (priv->empathy_group_cache);
+
       contacts = empathy_individual_manager_get_members (priv->manager);
 
       individual_store_members_changed_cb (priv->manager,
@@ -2012,6 +1944,14 @@ individual_store_get_individual_status_icon_with_icon_name (
           icon_name = g_strdup_printf ("%s-%s", status_icon_name,
               protocol_name);
         }
+      else
+        {
+          g_warning ("Cannot retrieve contact from individual '%s'",
+              folks_alias_details_get_alias (
+                FOLKS_ALIAS_DETAILS (individual)));
+
+          return NULL;
+        }
     }
   else
     {
