@@ -44,7 +44,6 @@
 #include <telepathy-glib/interfaces.h>
 
 #include "empathy-tp-file.h"
-#include "empathy-marshal.h"
 #include "empathy-time.h"
 #include "empathy-utils.h"
 
@@ -66,8 +65,6 @@
 /* EmpathyTpFile object */
 
 struct _EmpathyTpFilePrivate {
-  TpChannel *channel;
-
   GInputStream *in_stream;
   GOutputStream *out_stream;
 
@@ -78,7 +75,6 @@ struct _EmpathyTpFilePrivate {
   TpSocketAccessControl socket_access_control;
 
   /* transfer properties */
-  gboolean incoming;
   gint64 start_time;
   GArray *socket_address;
   guint port;
@@ -97,13 +93,7 @@ struct _EmpathyTpFilePrivate {
   gboolean is_closed;
 };
 
-enum {
-  PROP_0,
-  PROP_CHANNEL,
-  PROP_INCOMING
-};
-
-G_DEFINE_TYPE (EmpathyTpFile, empathy_tp_file, G_TYPE_OBJECT);
+G_DEFINE_TYPE (EmpathyTpFile, empathy_tp_file, TP_TYPE_FILE_TRANSFER_CHANNEL);
 
 /* private functions */
 
@@ -343,7 +333,7 @@ tp_file_start_transfer (EmpathyTpFile *self)
   if (self->priv->progress_callback != NULL)
     self->priv->progress_callback (self, 0, self->priv->progress_user_data);
 
-  if (self->priv->incoming)
+  if (!tp_channel_get_requested ((TpChannel *) self))
     {
       GInputStream *socket_stream;
 
@@ -429,7 +419,7 @@ tp_file_state_changed_cb (TpChannel *proxy,
       "old state = %u, state = %u, reason = %u\n"
       "\tincoming = %s, in_stream = %s, out_stream = %s",
       self->priv->state, state, reason,
-      self->priv->incoming ? "yes" : "no",
+      tp_channel_get_requested (proxy) ? "no" : "yes",
       self->priv->in_stream ? "present" : "not present",
       self->priv->out_stream ? "present" : "not present");
 
@@ -456,12 +446,13 @@ tp_file_state_changed_cb (TpChannel *proxy,
 }
 
 static void
-tp_file_transferred_bytes_changed_cb (TpChannel *proxy,
-    guint64 count,
-    gpointer user_data,
-    GObject *weak_object)
+tp_file_transferred_bytes_changed_cb (TpFileTransferChannel *channel,
+    GParamSpec *spec,
+    EmpathyTpFile *self)
 {
-  EmpathyTpFile *self = (EmpathyTpFile *) weak_object;
+  guint64 count;
+
+  count = tp_file_transfer_channel_get_transferred_bytes (channel);
 
   /* don't notify for 0 bytes count */
   if (count == 0)
@@ -469,8 +460,7 @@ tp_file_transferred_bytes_changed_cb (TpChannel *proxy,
 
   /* notify clients */
   if (self->priv->progress_callback != NULL)
-    self->priv->progress_callback (EMPATHY_TP_FILE (weak_object),
-        count, self->priv->progress_user_data);
+    self->priv->progress_callback (self, count, self->priv->progress_user_data);
 }
 
 static void
@@ -589,8 +579,7 @@ file_read_async_cb (GObject *source,
    */
   initialize_empty_ac_variant (self->priv->socket_access_control, &nothing);
 
-  tp_cli_channel_type_file_transfer_call_provide_file (
-      self->priv->channel, -1,
+  tp_cli_channel_type_file_transfer_call_provide_file ((TpChannel *) self, -1,
       self->priv->socket_address_type, self->priv->socket_access_control,
       &nothing, ft_operation_provide_or_accept_file_cb,
       NULL, NULL, G_OBJECT (self));
@@ -616,7 +605,7 @@ file_transfer_set_uri_cb (TpProxy *proxy,
    */
   initialize_empty_ac_variant (self->priv->socket_access_control, &nothing);
 
-  tp_cli_channel_type_file_transfer_call_accept_file (self->priv->channel,
+  tp_cli_channel_type_file_transfer_call_accept_file ((TpChannel *) self,
       -1, self->priv->socket_address_type, self->priv->socket_access_control,
       &nothing, self->priv->offset,
       ft_operation_provide_or_accept_file_cb, NULL, NULL, weak_object);
@@ -650,7 +639,7 @@ file_replace_async_cb (GObject *source,
   uri = g_file_get_uri (file);
   value = tp_g_value_slice_new_take_string (uri);
 
-  tp_cli_dbus_properties_call_set (self->priv->channel, -1,
+  tp_cli_dbus_properties_call_set ((TpChannel *) self, -1,
       TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, "URI", value,
       file_transfer_set_uri_cb, NULL, NULL, G_OBJECT (self));
 
@@ -682,7 +671,7 @@ close_channel_internal (EmpathyTpFile *self,
 
   self->priv->is_closing = TRUE;
 
-  tp_cli_channel_call_close (self->priv->channel, -1,
+  tp_cli_channel_call_close ((TpChannel *) self, -1,
     channel_closed_cb, GINT_TO_POINTER (cancel), NULL, G_OBJECT (self));
 }
 
@@ -700,13 +689,6 @@ do_dispose (GObject *object)
 {
   EmpathyTpFile *self = (EmpathyTpFile *) object;
 
-  if (self->priv->channel != NULL)
-    {
-      g_signal_handlers_disconnect_by_func (self->priv->channel,
-          tp_file_invalidated_cb, object);
-      tp_clear_object (&self->priv->channel);
-    }
-
   tp_clear_object (&self->priv->in_stream);
   tp_clear_object (&self->priv->out_stream);
   tp_clear_object (&self->priv->cancellable);
@@ -730,74 +712,33 @@ do_finalize (GObject *object)
   G_OBJECT_CLASS (empathy_tp_file_parent_class)->finalize (object);
 }
 
-static void
-do_get_property (GObject *object,
-    guint param_id,
-    GValue *value,
-    GParamSpec *pspec)
-{
-  EmpathyTpFile *self = (EmpathyTpFile *) object;
-
-  switch (param_id)
-    {
-      case PROP_CHANNEL:
-        g_value_set_object (value, self->priv->channel);
-        break;
-      case PROP_INCOMING:
-        g_value_set_boolean (value, self->priv->incoming);
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-        break;
-    };
-}
-
-static void
-do_set_property (GObject *object,
-    guint param_id,
-    const GValue *value,
-    GParamSpec *pspec)
-{
-  EmpathyTpFile *self = (EmpathyTpFile *) object;
-
-  switch (param_id)
-    {
-      case PROP_CHANNEL:
-        self->priv->channel = g_object_ref (g_value_get_object (value));
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-        break;
-    };
-}
-
 static void
 do_constructed (GObject *object)
 {
   EmpathyTpFile *self = (EmpathyTpFile *) object;
+  TpChannel *channel = (TpChannel *) self;
 
-  g_signal_connect (self->priv->channel, "invalidated",
+  g_signal_connect (self, "invalidated",
     G_CALLBACK (tp_file_invalidated_cb), self);
 
-  self->priv->incoming = !tp_channel_get_requested (self->priv->channel);
-
   tp_cli_channel_type_file_transfer_connect_to_file_transfer_state_changed (
-      self->priv->channel, tp_file_state_changed_cb, NULL, NULL, object, NULL);
+      channel, tp_file_state_changed_cb, NULL, NULL, object, NULL);
 
-  tp_cli_channel_type_file_transfer_connect_to_transferred_bytes_changed (
-      self->priv->channel, tp_file_transferred_bytes_changed_cb,
-      NULL, NULL, object, NULL);
+  g_signal_connect (self, "notify::transferred-bytes",
+    G_CALLBACK (tp_file_transferred_bytes_changed_cb), self);
 
-  tp_cli_dbus_properties_call_get (self->priv->channel,
+  tp_cli_dbus_properties_call_get (channel,
       -1, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, "State", tp_file_get_state_cb,
       NULL, NULL, object);
 
-  tp_cli_dbus_properties_call_get (self->priv->channel,
+  tp_cli_dbus_properties_call_get (channel,
       -1, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, "AvailableSocketTypes",
       tp_file_get_available_socket_types_cb, NULL, NULL, object);
 
   self->priv->state_change_reason =
       TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE;
+
+  G_OBJECT_CLASS (empathy_tp_file_parent_class)->constructed (object);
 }
 
 static void
@@ -808,66 +749,32 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
   object_class->finalize = do_finalize;
   object_class->dispose = do_dispose;
   object_class->constructed = do_constructed;
-  object_class->get_property = do_get_property;
-  object_class->set_property = do_set_property;
-
-  /* Construct-only properties */
-
-  /**
-   * EmpathyTpFile:channel:
-   *
-   * The #TpChannel requested for the file transfer.
-   */
-  g_object_class_install_property (object_class,
-      PROP_CHANNEL,
-      g_param_spec_object ("channel",
-          "telepathy channel",
-          "The file transfer channel",
-          TP_TYPE_CHANNEL,
-          G_PARAM_READWRITE |
-          G_PARAM_CONSTRUCT_ONLY));
-
-  /**
-   * EmpathyTpFile:incoming:
-   *
-   * %TRUE if the transfer is incoming, %FALSE if it's outgoing.
-   */
-  g_object_class_install_property (object_class,
-      PROP_INCOMING,
-      g_param_spec_boolean ("incoming",
-          "direction of transfer",
-          "The direction of the file being transferred",
-          FALSE,
-          G_PARAM_READABLE));
 
   g_type_class_add_private (object_class, sizeof (EmpathyTpFilePrivate));
 }
 
 /* public methods */
 
-/**
- * empathy_tp_file_new:
- * @channel: a #TpChannel
- * @incoming: whether the file transfer is incoming or not
- *
- * Creates a new #EmpathyTpFile wrapping @channel.
- * The returned #EmpathyTpFile should be unrefed
- * with g_object_unref() when finished with.
- *
- * Return value: a new #EmpathyTpFile
- */
 EmpathyTpFile *
-empathy_tp_file_new (TpChannel *channel)
+empathy_tp_file_new (TpSimpleClientFactory *factory,
+    TpConnection *conn,
+    const gchar *object_path,
+    const GHashTable *immutable_properties,
+    GError **error)
 {
-  EmpathyTpFile *self;
+  TpProxy *conn_proxy = (TpProxy *) conn;
 
-  g_return_val_if_fail (TP_IS_CHANNEL (channel), NULL);
+  g_return_val_if_fail (TP_IS_CONNECTION (conn), NULL);
+  g_return_val_if_fail (immutable_properties != NULL, NULL);
 
-  self = g_object_new (EMPATHY_TYPE_TP_FILE,
-      "channel", channel,
-      NULL);
-
-  return self;
+  return g_object_new (EMPATHY_TYPE_TP_FILE,
+           "factory", factory,
+           "connection", conn,
+           "dbus-daemon", conn_proxy->dbus_daemon,
+           "bus-name", conn_proxy->bus_name,
+           "object-path", object_path,
+           "channel-properties", immutable_properties,
+           NULL);
 }
 
 /**
@@ -956,22 +863,6 @@ empathy_tp_file_offer (EmpathyTpFile *self,
       file_read_async_cb, self);
 }
 
-/**
- * empathy_tp_file_is_incoming:
- * @self: an #EmpathyTpFile
- *
- * Returns whether @self is incoming.
- *
- * Return value: %TRUE if the @self is incoming, otherwise %FALSE
- */
-gboolean
-empathy_tp_file_is_incoming (EmpathyTpFile *self)
-{
-  g_return_val_if_fail (EMPATHY_IS_TP_FILE (self), FALSE);
-
-  return self->priv->incoming;
-}
-
 /**
  * empathy_tp_file_cancel:
  * @self: an #EmpathyTpFile
